@@ -2,7 +2,7 @@
 This file is part of the `arabluatex' package
 
 ArabLuaTeX -- Processing ArabTeX notation under LuaLaTeX
-Copyright (C) 2016--2018  Robert Alessi
+Copyright (C) 2016--2019  Robert Alessi
 
 Please send error reports and suggestions for improvements to Robert
 Alessi <alessi@robertalessi.net>
@@ -22,6 +22,8 @@ along with this program.  If not, see
 <http://www.gnu.org/licenses/>.
 --]]
 
+arabluatex = {}
+
 require("arabluatex_voc")
 require("arabluatex_fullvoc")
 require("arabluatex_novoc")
@@ -49,6 +51,8 @@ local cmdstar = lpeg.Cs(spce * lpeg.P("*"))
 local bsqbracketsii = lpeg.Cs(bsqbrackets^-2)
 local bcbracesii = lpeg.Cs(bcbraces^-2)
 local cmd = lpeg.Cs(dblbkslash * ascii^1 * cmdstar^-1)
+local rawcmd = lpeg.Cs(dblbkslash * ascii^1)
+local aftercmd = lpeg.Cs(lpeg.S("*[{,.?;:'`\"") + dblbkslash)
 local cmdargs = lpeg.Cs(spce^-1 * bsqbracketsii * bcbracesii * bsqbrackets^-1)
 local arbargs = lpeg.Cs(spce^-1 * bsqbrackets^-1 * bcbraces)
 local baytargs = lpeg.Cs(spce * bcbraces * bsqbrackets^-1 * bcbraces)
@@ -93,7 +97,7 @@ local albrkcmds = {
 }
 local brkcmds = {}
 
-function mkarbbreak(str, opt)
+function arabluatex.mkarbbreak(str, opt)
    str = str ..","
    str = string.gsub(str, "%s+", "")
    local fieldstart = 1
@@ -228,10 +232,11 @@ local function takeoutcapetc(str)
 			body = string.sub(body, 2, -2)
 			return string.format("%s", body)
    end)
-   str = string.gsub(str, "(\\uc.?)(%b{})", function(tag, body)
+   str = string.gsub(str, "(\\uc%s?)(%b{})", function(tag, body)
 			body = string.sub(body, 2, -2)
 			return string.format("%s", body)
    end)
+   str = string.gsub(str, "\\uc%s", "")
    str = string.gsub(str, "\\linebreak", "")
    str = string.gsub(str, "\\%-", "")
    return str
@@ -487,7 +492,7 @@ local function novoc(str)
    inside = indnum(inside)
    return string.format("\\arabicfont{}%s", inside)
    end)
-return str
+   return str
 end
 
 local function novoceasy(str)
@@ -523,7 +528,7 @@ local function novoceasy(str)
    inside = indnum(inside)
    return string.format("\\arabicfont{}%s", inside)
    end)
-return str
+   return str
 end
 
 local function transdmg(str, rules)
@@ -561,9 +566,9 @@ local function transdmg(str, rules)
    for i = 1,#nulltr do
       inside = string.gsub(inside, nulltr[i].a, nulltr[i].b)
    end
-   return string.format("\\txtrans{%s}", inside)
+   return string.format("%s", inside)
    end)
-return str
+   return str
 end
 
 local function transloc(str)
@@ -599,9 +604,9 @@ local function transloc(str)
    for i = 1,#nulltr do
       inside = string.gsub(inside, nulltr[i].a, nulltr[i].b)
    end
-   return string.format("\\txtrans{%s}", inside)
+   return string.format("%s", inside)
    end)
-return str
+   return str
 end
 
 local function transarabica(str)
@@ -634,9 +639,9 @@ local function transarabica(str)
    for i = 1,#nulltr do
       inside = string.gsub(inside, nulltr[i].a, nulltr[i].b)
    end
-   return string.format("\\txtrans{%s}", inside)
+   return string.format("%s", inside)
    end)
-return str
+   return str
 end
 
 local function processbuckw(str)
@@ -647,7 +652,7 @@ local function processbuckw(str)
    end
    return string.format("\\arb{%s}", inside)
    end)
-return str
+   return str
 end
 
 -- The following functions produce a copy of the original .tex source
@@ -656,17 +661,17 @@ end
 local utffilesuffix = "_out"
 local export_utf = "no"
 
-function al_utffilesuffix(str)
+function arabluatex.utffilesuffix(str)
    utffilesuffix = str
    return true
 end
 
-function al_doexport(str)
+function arabluatex.doexport(str)
    export_utf = str
    return true
 end
 
-function al_openstream()
+function arabluatex.openstream()
    local f = io.open(tex.jobname..utffilesuffix.."_tmp.tex", "a+")
    local preamble = io.open(tex.jobname..".tex", "r")
    for line in preamble:lines() do
@@ -703,8 +708,8 @@ local function processarbtoutf(str)
 			body = string.gsub(body, "(\\abjad%s?)(%b{})", function(btag, bbody)
 					      bbody = string.sub(bbody, 2, -2)
 					      if tonumber(bbody) ~= nil then
-						 bbody = abjadify(bbody)
-						 return string.format("\\aemph{\\arb[novoc]{%s}}", bbody)
+						 bbody = arabluatex.abjadify(bbody)
+						 return string.format("\\oline*{\\arb[novoc]{%s}}", bbody)
 					      else
 						 return string.format("%s{%s}", btag, bbody)
 					      end
@@ -790,7 +795,7 @@ local function processarbtoutf(str)
    return str
 end
 
-function arbtoutf(str)
+function arabluatex.arbtoutf(str)
    str = processarbtoutf(str)
    str = "\\ArbOutFile{"..str.."}"
    str = string.gsub(str, "(\\ArbOutFile)%s?(%b{})", function(tag, body)
@@ -811,7 +816,7 @@ function arbtoutf(str)
    return str
 end
 
-function tooutfile(str, nl)
+function arabluatex.tooutfile(str, nl)
    local f = io.open(tex.jobname..utffilesuffix.."_tmp.tex", "a+")
    if nl == "newline" then
       f:write(str, "\n\n")
@@ -822,7 +827,7 @@ function tooutfile(str, nl)
    return str
 end
 
-function al_closestream()
+function arabluatex.closestream()
    local f = io.open(tex.jobname..utffilesuffix.."_tmp.tex", "r")
    local o = io.open(tex.jobname..utffilesuffix..".tex", "w")
    local t = f:read("*a")
@@ -869,6 +874,7 @@ function al_closestream()
 	 [[ and compile it one more time}}]])
       --
    else end
+   t = gsub(t, rawcmd * spce^1 * aftercmd, "%1%3")
    t = t.."\n\\end{document}"
    io.write(t)
    o:write(t)
@@ -879,7 +885,7 @@ function al_closestream()
 end
 
 -- Process standard arabluatex modes:
-function processvoc(str, rules, scheme)
+function arabluatex.processvoc(str, rules, scheme)
    str = takeoutarb(str)
    str = processarbnull(str, scheme)
    str = takeoutcapetc(str)
@@ -897,17 +903,17 @@ function processvoc(str, rules, scheme)
    str = unprotectarb(str)
    if export_utf == "yes" then
       tofile = "\\txarb{"..str.."}"
-      tooutfile(tofile)
+      arabluatex.tooutfile(tofile)
    elseif export_utf == "arabverse" then
       tofile = "\\txarb{"..str.."}"
-      tooutfile(tofile)
+      arabluatex.tooutfile(tofile)
    else
       return str
    end
    return ""
 end
 
-function processfullvoc(str, rules, scheme)
+function arabluatex.processfullvoc(str, rules, scheme)
    str = takeoutarb(str)
    str = processarbnull(str, scheme)
    str = takeoutcapetc(str)
@@ -927,17 +933,17 @@ function processfullvoc(str, rules, scheme)
    str = unprotectarb(str)
    if export_utf == "yes" then
       tofile = "\\txarb{"..str.."}"
-      tooutfile(tofile)
+      arabluatex.tooutfile(tofile)
    elseif export_utf == "arabverse" then
       tofile = "\\txarb{"..str.."}"
-      tooutfile(tofile)
+      arabluatex.tooutfile(tofile)
    else
       return str
    end
    return ""
 end
 
-function processnovoc(str, rules, scheme)
+function arabluatex.processnovoc(str, rules, scheme)
    str = takeoutarb(str)
    str = processarbnull(str, scheme)
    str = takeoutcapetc(str)
@@ -955,17 +961,17 @@ function processnovoc(str, rules, scheme)
    str = unprotectarb(str)
    if export_utf == "yes" then
       tofile = "\\txarb{"..str.."}"
-      tooutfile(tofile)
+      arabluatex.tooutfile(tofile)
    elseif export_utf == "arabverse" then
       tofile = "\\txarb{"..str.."}"
-      tooutfile(tofile)
+      arabluatex.tooutfile(tofile)
    else
       return str
    end
    return ""
 end
 
-function processtrans(str, mode, rules, scheme)
+function arabluatex.processtrans(str, mode, rules, scheme)
    str = takeoutarb(str)
    str = processdiscretionary(str)
    str = processarbnull(str, scheme)
@@ -985,18 +991,18 @@ function processtrans(str, mode, rules, scheme)
    end
    str = unprotectarb(str)
    if export_utf == "yes" then
-      tofile = str
-      tooutfile(tofile)
+      tofile = "\\txtrans{"..str.."}"
+      arabluatex.tooutfile(tofile)
    elseif export_utf == "arabverse" then
-      tofile = str
-      tooutfile(tofile)
+      tofile = "\\txtrans{"..str.."}"
+      arabluatex.tooutfile(tofile)
    else
       return str
    end
    return ""
 end
 
-function newarbmark(abbr, rtlmk, ltrmk)
+function arabluatex.newarbmark(abbr, rtlmk, ltrmk)
    abbr = "@"..abbr
    rtlmk = "\\arabicfont{}"..rtlmk
    table.insert(arbmarks, {a = abbr, b = rtlmk, c = ltrmk})
@@ -1013,7 +1019,7 @@ local function isintable(table, element)
    return false
 end
 
-function processarbmarks(str, dir)
+function arabluatex.processarbmarks(str, dir)
    str = "@"..str
    if not isintable(arbmarks, str) then
       str = "\\LR{<??>}"..atletter.."\\al@wrong@mark{}"..atother
@@ -1038,21 +1044,24 @@ function processarbmarks(str, dir)
    end
    if export_utf == "yes" then
       tofile = str
-      tooutfile(tofile)
+      arabluatex.tooutfile(tofile)
    elseif export_utf == "arabverse" then
       tofile = str
-      tooutfile(tofile)
+      arabluatex.tooutfile(tofile)
    else
       return str
    end
    return ""
 end
 
-function uc(str)
+function arabluatex.uc(str)
    str = string.gsub(str, "(\\txtrans.?)(%b{})", function(tag, body)
 			body = string.sub(body, 2, -2)
 			return string.format("%s", body)
    end)
+   str = string.gsub(str, "{", "\\@al@ob")
+   str = string.gsub(str, "} ", "\\@al@cb@sp ")
+   str = string.gsub(str, "}", "\\@al@cb")
    -- Allah and ibn
    str = string.gsub(str, "(al%-lāh)([uai]?)", "{Allāh%2}")
    str = string.gsub(str, "([%'%-]?)(l%-lāh)([uai]?)", "%1{Llāh%3}")
@@ -1099,14 +1108,19 @@ function uc(str)
    for i = 1,#lcuc do
       str = string.gsub(str, "(%s[%(%<%[]?)"..lcuc[i].a, "%1"..lcuc[i].b)
    end
+   str = string.gsub(str, "{", "")
+   str = string.gsub(str, "}", "")
+   str = string.gsub(str, "\\@al@ob", "{")
+   str = string.gsub(str, "\\@al@cb@sp ", "} ")
+   str = string.gsub(str, "\\@al@cb", "}")
    if export_utf == "yes" then
-      tofile = "\\txtrans{"..str.."}"
-      tooutfile(tofile)
+      tofile = str
+      arabluatex.tooutfile(tofile)
    elseif export_utf == "arabverse" then
-      tofile = "\\txtrans{"..str.."}"
-      tooutfile(tofile)
+      tofile = str
+      arabluatex.tooutfile(tofile)
    else
-      return "\\txtrans{"..str.."}"
+      return str
    end
    return ""
 end
@@ -1114,7 +1128,7 @@ end
 -- this function is adapted from an 'obsolete project' of Khaled
 -- Hosny's that dates back to 2010. Thanks to him.
 -- See https://github.com/khaledhosny/lualatex-arabic
-function abjadify(n)
+function arabluatex.abjadify(n)
    local abjadnum = ""
    n = tonumber(n)
     if n >= 1000 then
@@ -1137,7 +1151,7 @@ function abjadify(n)
     return "\\arb[novoc]{"..abjadnum.."}"
 end
 
-function abraces(str)
+function arabluatex.abraces(str)
    if tex.textdir == "TRT" then
       str = "\\}"..str.."\\{"
    elseif tex.textdir == "TLT" then
@@ -1146,20 +1160,20 @@ function abraces(str)
    return str
 end
 
-function aemph(str, opt)
+function arabluatex.aemph(str, opt)
    if tex.textdir == "TRT" then
-      str = "$\\overline{\\text{\\textdir TRT{}"..str.."}}$"
+      str = "\\oline{\\textdir TRT{}"..str.."}"
    elseif tex.textdir == "TLT" then
       if opt == "over" then
-	 str = "$\\overline{\\text{"..str.."}}$"
+	 str = "\\oline{"..str.."}"
       else
-	 str = "$\\underline{\\text{"..str.."}}$"
+	 str = "\\uline{"..str.."}"
       end
    end
    return str
 end
 
-function ayah(str)
+function arabluatex.ayah(str)
    if tonumber(str) ~= nil and str.len(str) < 4 then
       if tex.textdir == "TRT" then
 	 str = indnum(str).."^^^^06dd"
